rm(list=ls())
library(hdm)
#library(hdmrw) ### use implementation in hdm
library(mvtnorm)
library(knockoff)
library(foreach)

# for parallelization
library(parallel)
library(doParallel)

wspace = "## PASTE YOUR WORKING SPACE HERE ###"
setwd(wspace)

date <- Sys.Date()
date

sessionInfo()

source(paste0(wspace, "/DGP_s6.R"))

R = 2000

# correlation for Sigma 

# setting: "medium"
rho = 0.8

# Setting for x-dependent choice of lambda with rlasso (set TRUE in last version)
X.dep = TRUE

# FDR level for knockoffs (to be specified before)
gamma_knockoff <- 0.1 

n = 200
p = 60

# Parallelized loop 
ncores <- detectCores() - 1
cl <- makeCluster(ncores)
registerDoParallel(cl)

begin = Sys.time()
begin

set.seed(14141)

res <- foreach (r = seq(R), .packages = c("hdm", "mvtnorm", "knockoff", "foreach")) %dopar% {
  #generate data
  
  set.seed(1234567 + as.numeric(r))
  # print(j)
  DGP1 <- DGP_desc5(n,p, betamax = 9, decay = 0.99, threshold = 0.75, noisevar = 3, rho = rho)
  y <- DGP1$data$y
  x <- as.matrix(DGP1$data[,-1,drop=F])
  beta <- DGP1$beta
  
  #rl = rlassoEffects(x,y, index = c(1:p), method = "double selection")
  rl = rlassoEffects(x,y, index = c(1:p), penalty = list(homoscedastic = TRUE,   X.dependent.lambda = X.dep), method = "double selection")
  
  coef_rl <- coef(rl)
  # ols = lm(y ~-1 + x)
  
  set.seed(123 + as.numeric(r))
  pvals1 <- p_adjust(rl, method = "RW", stepdown = TRUE)[,2]
  
  # set.seed(123 + as.numeric(j))
  #  pvals2[j,] <- p_adjust(rl, method = "RW", stepdown = FALSE)[,2]
  
  pvals3 <- p_adjust(rl, method = "BH")[,2]
  pvals4 <- p_adjust(rl, method = "bonferroni")[,2]
  pvals5 <- p_adjust(rl, method = "holm")[,2]
  pvals6 <- p_adjust(rl, method = "none")[,2]
 
  # Knockoffs
  
  # setting with 0.9 correlation does not work for knockoffs; 0.75 
  
  # 0) default (default should be sdp-2nd-order gaussian knockoffs for n < 500)
  knockoff1 = knockoff.filter(x, y)
  
  # 1) fixed
    # settings as in fixed design vignette
  knockoff2 = knockoff.filter(x, y, knockoffs = create.fixed, statistic = stat.glmnet_lambdasmax, fdr = gamma_knockoff)
  
  # 2) model-X (assume Sigma is known)
  gaussian_knockoffs = function(x) create.gaussian(x, rep(0,p), DGP1$covar)
  knockoff3 = knockoff.filter(x, y, knockoffs = gaussian_knockoffs)
  
  # 3) model-x with asdp 
  gaussian_knockoffs_asdp = function(x) create.gaussian(x, method = "asdp")
  knockoff4 = knockoff.filter(x, y, knockoffs = gaussian_knockoffs_asdp)
  
  # 4) model-x with equi correlated knockoffs
  # gaussian_knockoffs_equi = function(x) create.gaussian(x, method = "equi")
  # knockoff5 = knockoff.filter(x, y, knockoffs = gaussian_knockoffs_equi)
  # 


  result = list("beta_0" = beta, "coef_rl" = coef_rl, "pvals_RW" = pvals1, 
                "pvals_BH" = pvals3, "pvals_bonf" = pvals4, "pvals_holm" = pvals5, "pvals_none" = pvals6, 
                "knockoff_default" = knockoff1, "knockoff_fixed" = knockoff2, "knockoff_gaussian" = knockoff3, 
                "knockoff4" = knockoff4) #, "knockoff5" = knockoff5)
  
}

end = Sys.time()
time = end - begin
time

# save results
save(res , file = paste0(wspace, "/results/", date, "_results_medium2_cor_n_", n, "_p_", p, "_R_", R, ".RData"))

stopCluster(cl)